@@ -1,12 +1,14 @@
-"""GNUmed patient EMR tree browser.
-"""
+"""GNUmed patient EMR tree browser."""
 #================================================================
-__version__ = "$Revision: 1.111 $"
 __author__ = "cfmoro1976@yahoo.es, sjtan@swiftdsl.com.au, Karsten.Hilbert@gmx.net"
-__license__ = "GPL"
+__license__ = "GPL v2 or later"
 
 # std lib
-import sys, os.path, StringIO, codecs, logging
+import sys
+import os.path
+import StringIO
+import codecs
+import logging
 
 
 # 3rd party
@@ -14,9 +16,20 @@ import wx
 
 
 # GNUmed libs
-from Gnumed.pycommon import gmI18N, gmDispatcher, gmExceptions, gmTools
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmDispatcher
+from Gnumed.pycommon import gmExceptions
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDateTime
+from Gnumed.pycommon import gmLog2
+
 from Gnumed.exporters import gmPatientExporter
-from Gnumed.business import gmEMRStructItems, gmPerson, gmSOAPimporter, gmPersonSearch
+
+from Gnumed.business import gmEMRStructItems
+from Gnumed.business import gmPerson
+from Gnumed.business import gmSOAPimporter
+from Gnumed.business import gmPersonSearch
+
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmEMRStructWidgets
 from Gnumed.wxpython import gmSOAPWidgets
@@ -29,7 +42,6 @@ from Gnumed.wxpython import gmFamilyHistoryWidgets
 
 
 _log = logging.getLogger('gm.ui')
-_log.info(__version__)
 
 #============================================================
 def export_emr_to_ascii(parent=None):
@@ -68,7 +80,6 @@ def export_emr_to_ascii(parent=None):
 
 	_log.debug('exporting EMR to [%s]', fname)
 
-#	output_file = open(fname, 'wb')
 	output_file = codecs.open(fname, 'wb', encoding='utf8', errors='replace')
 	exporter = gmPatientExporter.cEmrExport(patient = pat)
 	exporter.set_output_file(output_file)
@@ -93,12 +104,14 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 
 		gmGuiHelpers.cTreeExpansionHistoryMixin.__init__(self)
 
-		self.__details_display = None
-		self.__details_display_mode = u'details'				# "details" or "journal"
-		self.__enable_display_mode_selection = None
+		self.__soap_display = None
+		self.__soap_display_mode = u'details'				# "details" or "journal"
+		self.__img_display = None
+		self.__cb__enable_display_mode_selection = lambda x:x
+		self.__cb__select_edit_mode = lambda x:x
+		self.__cb__add_soap_editor = lambda x:x
 		self.__pat = gmPerson.gmCurrentPatient()
 		self.__curr_node = None
-		self.__exporter = gmPatientExporter.cEmrExport(patient = self.__pat)
 
 		self._old_cursor_pos = None
 
@@ -117,17 +130,45 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 
 		return True
 	#--------------------------------------------------------
-	def set_narrative_display(self, narrative_display=None):
-		self.__details_display = narrative_display
+	def _get_soap_display(self):
+		return self.__soap_display
+
+	def _set_soap_display(self, soap_display=None):
+		self.__soap_display = soap_display
+
+	soap_display = property(_get_soap_display, _set_soap_display)
 	#--------------------------------------------------------
-	def set_image_display(self, image_display=None):
+	def _get_image_display(self):
+		return self.__img_display
+
+	def _set_image_display(self, image_display=None):
 		self.__img_display = image_display
+
+	image_display = property(_get_image_display, _set_image_display)
 	#--------------------------------------------------------
 	def set_enable_display_mode_selection_callback(self, callback):
 		if not callable(callback):
 			raise ValueError('callback [%s] not callable' % callback)
+		self.__cb__enable_display_mode_selection = callback
+	#--------------------------------------------------------
+	def _set_edit_mode_selector(self, callback):
+		if callback is None:
+			callback = lambda x:x
+		if not callable(callback):
+			raise ValueError('edit mode selector [%s] not callable' % callback)
+		self.__cb__select_edit_mode = callback
 
-		self.__enable_display_mode_selection = callback
+	edit_mode_selector = property(lambda x:x, _set_edit_mode_selector)
+	#--------------------------------------------------------
+	def _set_soap_editor_adder(self, callback):
+		if callback is None:
+			callback = lambda x:x
+		if not callable(callback):
+			raise ValueError('soap editor adder [%s] not callable' % callback)
+		self.__cb__add_soap_editor = callback
+
+	soap_editor_adder = property(lambda x:x, _set_soap_editor_adder)
+	#--------------------------------------------------------
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
@@ -135,6 +176,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		"""Configures enabled event signals."""
 		wx.EVT_TREE_SEL_CHANGED (self, self.GetId(), self._on_tree_item_selected)
 		wx.EVT_TREE_ITEM_RIGHT_CLICK (self, self.GetId(), self._on_tree_item_right_clicked)
+		self.Bind(wx.EVT_TREE_ITEM_EXPANDING, self._on_tree_item_expanding)
 
 		# handle tooltips
 #		wx.EVT_MOTION(self, self._on_mouse_motion)
@@ -145,20 +187,38 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		gmDispatcher.connect(signal = 'health_issue_mod_db', receiver = self._on_issue_mod_db)
 		gmDispatcher.connect(signal = 'family_history_mod_db', receiver = self._on_issue_mod_db)
 	#--------------------------------------------------------
+	def clear_tree(self):
+		self.DeleteAllItems()
+	#--------------------------------------------------------
 	def __populate_tree(self):
 		"""Updates EMR browser data."""
 		# FIXME: auto select the previously self.__curr_node if not None
 		# FIXME: error handling
 
+		if not self.__pat.connected:
+			return
+
 		wx.BeginBusyCursor()
-
 #		self.snapshot_expansion()
-
 		# init new tree
+		root_item = self.__populate_root_node()
+#		self.__exporter.get_historical_tree(self)				# this is slow
+		self.__curr_node = root_item
+		self.SelectItem(root_item)
+		self.Expand(root_item)
+		self.__update_text_for_selected_node()					# this is fairly slow, too
+#		self.restore_expansion()
+		wx.EndBusyCursor()
+		return True
+	#--------------------------------------------------------
+	def __populate_root_node(self):
+
 		self.DeleteAllItems()
+
 		root_item = self.AddRoot(_('EMR of %(lastnames)s, %(firstnames)s') % self.__pat.get_active_name())
 		self.SetItemPyData(root_item, None)
 		self.SetItemHasChildren(root_item, True)
+
 		self.__root_tooltip = self.__pat['description_gender'] + u'\n'
 		if self.__pat['deceased'] is None:
 			self.__root_tooltip += u' %s (%s)\n\n' % (
@@ -169,7 +229,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 			template = u' %s - %s (%s)\n\n'
 			self.__root_tooltip += template % (
 				self.__pat.get_formatted_dob(format = '%d.%b %Y', encoding = gmI18N.get_encoding()),
-				self.__pat['deceased'].strftime('%d.%b %Y').decode(gmI18N.get_encoding()),
+				gmDateTime.pydt_strftime(self.__pat['deceased'], '%Y %b %d'),
 				self.__pat['medical_age']
 			)
 		self.__root_tooltip += gmTools.coalesce(self.__pat['comment'], u'', u'%s\n\n')
@@ -199,23 +259,12 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		if self.__root_tooltip == u'':
 			self.__root_tooltip = u' '
 
-		# have the tree filled by the exporter
-		self.__exporter.get_historical_tree(self)
-		self.__curr_node = root_item
-
-		self.SelectItem(root_item)
-		self.Expand(root_item)
-		self.__update_text_for_selected_node()
-
-#		self.restore_expansion()
-
-		wx.EndBusyCursor()
-		return True
+		return root_item
 	#--------------------------------------------------------
 	def __update_text_for_selected_node(self):
 		"""Displays information for the selected tree node."""
 
-		if self.__details_display is None:
+		if self.__soap_display is None:
 			self.__img_display.clear()
 			return
 
@@ -223,12 +272,25 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 			self.__img_display.clear()
 			return
 
-		node_data = self.GetPyData(self.__curr_node)
+		if not self.__curr_node.IsOk():
+			return
+
+		try:
+			node_data = self.GetPyData(self.__curr_node)
+		except wx.PyAssertionError:
+			node_data = None		# fake a root node
+			_log.exception('unfathomable self.GetPyData() problem occurred, faking root node')
+			_log.debug('real node: %s', self.__curr_node)
+			_log.debug('node.IsOk(): %s', self.__curr_node.IsOk())		# already survived this further up
+			_log.debug('is root node: %s', self.__curr_node == self.GetRootItem())
+			_log.debug('node.m_pItem: %s', getattr(self.__curr_node.m_pItem), '<NO SUCH ATTRIBUTE>')
+			_log.debug('node attributes: %s', dir(self.__curr_node))
+			gmLog2.log_stack_trace()
 		doc_folder = self.__pat.get_document_folder()
 
 		if isinstance(node_data, gmEMRStructItems.cHealthIssue):
-			self.__enable_display_mode_selection(True)
-			if self.__details_display_mode == u'details':
+			self.__cb__enable_display_mode_selection(True)
+			if self.__soap_display_mode == u'details':
 				txt = node_data.format(left_margin=1, patient = self.__pat)
 			else:
 				txt = node_data.format_as_journal(left_margin = 1)
@@ -239,14 +301,14 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 			)
 
 		elif isinstance(node_data, type({})):
-			self.__enable_display_mode_selection(False)
+			self.__cb__enable_display_mode_selection(False)
 			# FIXME: turn into real dummy issue
 			txt = _('Pool of unassociated episodes:\n\n  "%s"') % node_data['description']
 			self.__img_display.clear()
 
 		elif isinstance(node_data, gmEMRStructItems.cEpisode):
-			self.__enable_display_mode_selection(True)
-			if self.__details_display_mode == u'details':
+			self.__cb__enable_display_mode_selection(True)
+			if self.__soap_display_mode == u'details':
 				txt = node_data.format(left_margin = 1, patient = self.__pat)
 			else:
 				txt = node_data.format_as_journal(left_margin = 1)
@@ -256,7 +318,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 			)
 
 		elif isinstance(node_data, gmEMRStructItems.cEncounter):
-			self.__enable_display_mode_selection(False)
+			self.__cb__enable_display_mode_selection(False)
 			epi = self.GetPyData(self.GetItemParent(self.__curr_node))
 			txt = node_data.format (
 				episodes = [epi['pk_episode']],
@@ -273,75 +335,20 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 
 		# root node == EMR level
 		else:
-			self.__enable_display_mode_selection(False)
-			emr = self.__pat.get_emr()
-			txt = emr.format_summary(dob = self.__pat['dob'])
+			self.__cb__enable_display_mode_selection(True)
+			if self.__soap_display_mode == u'details':
+				emr = self.__pat.get_emr()
+				txt = emr.format_summary(dob = self.__pat['dob'])
+			else:
+				txt = self.__pat.emr.format_as_journal(left_margin = 1, patient = self.__pat)
 			self.__img_display.clear()
 
-		self.__details_display.Clear()
-		self.__details_display.WriteText(txt)
-		self.__details_display.ShowPosition(0)
+		self.__soap_display.Clear()
+		self.__soap_display.WriteText(txt)
+		self.__soap_display.ShowPosition(0)
 	#--------------------------------------------------------
 	def __make_popup_menus(self):
 
-		# - episodes
-		self.__epi_context_popup = wx.Menu(title = _('Episode Actions:'))
-
-		menu_id = wx.NewId()
-		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Edit details')))
-		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__edit_episode)
-
-		menu_id = wx.NewId()
-		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Delete')))
-		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__delete_episode)
-
-		menu_id = wx.NewId()
-		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Promote')))
-		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__promote_episode_to_issue)
-
-		menu_id = wx.NewId()
-		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Move encounters')))
-		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__move_encounters)
-
-		# - encounters
-		self.__enc_context_popup = wx.Menu(title = _('Encounter Actions:'))
-		# - move data
-		menu_id = wx.NewId()
-		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('Move data to another episode')))
-		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__relink_encounter_data2episode)
-		# - edit encounter details
-		menu_id = wx.NewId()
-		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('Edit details')))
-		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__edit_encounter_details)
-
-		item = self.__enc_context_popup.Append(-1, _('Edit progress notes'))
-		self.Bind(wx.EVT_MENU, self.__edit_progress_notes, item)
-
-		item = self.__enc_context_popup.Append(-1, _('Move progress notes'))
-		self.Bind(wx.EVT_MENU, self.__move_progress_notes, item)
-
-		item = self.__enc_context_popup.Append(-1, _('Export for Medistar'))
-		self.Bind(wx.EVT_MENU, self.__export_encounter_for_medistar, item)
-
-		# - health issues
-		self.__issue_context_popup = wx.Menu(title = _('Health Issue Actions:'))
-
-		menu_id = wx.NewId()
-		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Edit details')))
-		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__edit_issue)
-
-		menu_id = wx.NewId()
-		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Delete')))
-		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__delete_issue)
-
-		self.__issue_context_popup.AppendSeparator()
-
-		menu_id = wx.NewId()
-		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Open to encounter level')))
-		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__expand_issue_to_encounter_level)
-		# print " attach issue to another patient"
-		# print " move all episodes to another issue"
-
 		# - root node
 		self.__root_context_popup = wx.Menu(title = _('EMR Actions:'))
 
@@ -352,6 +359,9 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		item = self.__root_context_popup.Append(-1, _('Create episode'))
 		self.Bind(wx.EVT_MENU, self.__create_episode, item)
 
+		item = self.__root_context_popup.Append(-1, _('Create progress note'))
+		self.Bind(wx.EVT_MENU, self.__create_soap_editor, item)
+
 		menu_id = wx.NewId()
 		self.__root_context_popup.AppendItem(wx.MenuItem(self.__root_context_popup, menu_id, _('Manage allergies')))
 		wx.EVT_MENU(self.__root_context_popup, menu_id, self.__document_allergy)
@@ -393,6 +403,74 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		menu_id = wx.NewId()
 		expand_menu.AppendItem(wx.MenuItem(expand_menu, menu_id, _('... encounter level')))
 		wx.EVT_MENU(expand_menu, menu_id, self.__expand_to_encounter_level)
+
+		# - health issues
+		self.__issue_context_popup = wx.Menu(title = _('Health Issue Actions:'))
+
+		menu_id = wx.NewId()
+		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Edit details')))
+		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__edit_issue)
+
+		menu_id = wx.NewId()
+		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Delete')))
+		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__delete_issue)
+
+		self.__issue_context_popup.AppendSeparator()
+
+		menu_id = wx.NewId()
+		self.__issue_context_popup.AppendItem(wx.MenuItem(self.__issue_context_popup, menu_id, _('Open to encounter level')))
+		wx.EVT_MENU(self.__issue_context_popup, menu_id, self.__expand_issue_to_encounter_level)
+		# print " attach issue to another patient"
+		# print " move all episodes to another issue"
+
+		item = self.__issue_context_popup.Append(-1, _('Create progress note'))
+		self.Bind(wx.EVT_MENU, self.__create_soap_editor, item)
+
+		# - episodes
+		self.__epi_context_popup = wx.Menu(title = _('Episode Actions:'))
+
+		menu_id = wx.NewId()
+		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Edit details')))
+		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__edit_episode)
+
+		menu_id = wx.NewId()
+		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Delete')))
+		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__delete_episode)
+
+		menu_id = wx.NewId()
+		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Promote')))
+		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__promote_episode_to_issue)
+
+		item = self.__epi_context_popup.Append(-1, _('Create progress note'))
+		self.Bind(wx.EVT_MENU, self.__create_soap_editor, item)
+
+		menu_id = wx.NewId()
+		self.__epi_context_popup.AppendItem(wx.MenuItem(self.__epi_context_popup, menu_id, _('Move encounters')))
+		wx.EVT_MENU(self.__epi_context_popup, menu_id, self.__move_encounters)
+
+		# - encounters
+		self.__enc_context_popup = wx.Menu(title = _('Encounter Actions:'))
+		# - move data
+		menu_id = wx.NewId()
+		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('Move data to another episode')))
+		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__relink_encounter_data2episode)
+		# - edit encounter details
+		menu_id = wx.NewId()
+		self.__enc_context_popup.AppendItem(wx.MenuItem(self.__enc_context_popup, menu_id, _('Edit details')))
+		wx.EVT_MENU(self.__enc_context_popup, menu_id, self.__edit_encounter_details)
+
+		# would require pre-configurable save-under which we don't have
+		#item = self.__enc_context_popup.Append(-1, _('Create progress note'))
+		#self.Bind(wx.EVT_MENU, self.__create_soap_editor, item)
+
+		item = self.__enc_context_popup.Append(-1, _('Edit progress notes'))
+		self.Bind(wx.EVT_MENU, self.__edit_progress_notes, item)
+
+		item = self.__enc_context_popup.Append(-1, _('Move progress notes'))
+		self.Bind(wx.EVT_MENU, self.__move_progress_notes, item)
+
+		item = self.__enc_context_popup.Append(-1, _('Export for Medistar'))
+		self.Bind(wx.EVT_MENU, self.__export_encounter_for_medistar, item)
 	#--------------------------------------------------------
 	def __handle_root_context(self, pos=wx.DefaultPosition):
 		self.PopupMenu(self.__root_context_popup, pos)
@@ -445,11 +523,47 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 		if result != wx.ID_YES:
 			return
 
-		try:
-			gmEMRStructItems.delete_episode(episode = self.__curr_node_data)
-		except gmExceptions.DatabaseObjectInUseError:
+		if not gmEMRStructItems.delete_episode(episode = self.__curr_node_data):
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete episode. There is still clinical data recorded for it.'))
+	#--------------------------------------------------------
+	def __expand_episode_node(self, episode_node=None):
+		self.DeleteChildren(episode_node)
+
+		emr = self.__pat.emr
+		epi = self.GetPyData(episode_node)
+		encounters = emr.get_encounters(episodes = [epi['pk_episode']], skip_empty = True)
+		if len(encounters) == 0:
+			self.SetItemHasChildren(episode_node, False)
 			return
+
+		self.SetItemHasChildren(episode_node, True)
+
+		for enc in encounters:
+			label = u'%s: %s' % (
+				enc['started'].strftime('%Y-%m-%d'),
+				gmTools.unwrap (
+					gmTools.coalesce (
+						gmTools.coalesce (
+							gmTools.coalesce (
+								enc.get_latest_soap (				# soAp
+									soap_cat = 'a',
+									episode = epi['pk_episode']
+								),
+								enc['assessment_of_encounter']		# or AOE
+							),
+							enc['reason_for_encounter']				# or RFE
+						),
+						enc['l10n_type']							# or type
+					),
+					max_length = 40
+				)
+			)
+			encounter_node = self.AppendItem(episode_node, label)
+			self.SetItemPyData(encounter_node, enc)
+			# we don't expand encounter nodes (what for ?)
+			self.SetItemHasChildren(encounter_node, False)
+
+		self.SortChildren(episode_node)
 	#--------------------------------------------------------
 	# encounter level
 	#--------------------------------------------------------
@@ -542,6 +656,47 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 			self.Expand(epi)
 			epi, epi_cookie = self.GetNextChild(self.__curr_node, epi_cookie)
 	#--------------------------------------------------------
+	def __expand_issue_node(self, issue_node=None):
+		self.DeleteChildren(issue_node)
+
+		emr = self.__pat.emr
+		issue = self.GetPyData(issue_node)
+		episodes = emr.get_episodes(issues = [issue['pk_health_issue']])
+		if len(episodes) == 0:
+			self.SetItemHasChildren(issue_node, False)
+			return
+
+		self.SetItemHasChildren(issue_node, True)
+
+		for episode in episodes:
+			episode_node =  self.AppendItem(issue_node, episode['description'])
+			self.SetItemPyData(episode_node, episode)
+			# fake it so we can expand it
+			self.SetItemHasChildren(episode_node, True)
+
+		self.SortChildren(issue_node)
+	#--------------------------------------------------------
+	def __expand_pseudo_issue_node(self, fake_issue_node=None):
+		self.DeleteChildren(fake_issue_node)
+
+		emr = self.__pat.emr
+		episodes = emr.unlinked_episodes
+		if len(episodes) == 0:
+			self.SetItemHasChildren(fake_issue_node, False)
+			return
+
+		self.SetItemHasChildren(fake_issue_node, True)
+
+		for episode in episodes:
+			episode_node =  self.AppendItem(fake_issue_node, episode['description'])
+			self.SetItemPyData(episode_node, episode)
+			if episode['episode_open']:
+				self.SetItemBold(fake_issue_node, True)
+			# fake it so we can expand it
+			self.SetItemHasChildren(episode_node, True)
+
+		self.SortChildren(fake_issue_node)
+	#--------------------------------------------------------
 	# EMR level
 	#--------------------------------------------------------
 	def __create_issue(self, event):
@@ -550,6 +705,10 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 	def __create_episode(self, event):
 		gmEMRStructWidgets.edit_episode(parent = self, episode = None)
 	#--------------------------------------------------------
+	def __create_soap_editor(self, event):
+		self.__cb__select_edit_mode(True)
+		self.__cb__add_soap_editor(problem = self.__curr_node_data, allow_same_problem = False)
+	#--------------------------------------------------------
 	def __document_allergy(self, event):
 		dlg = gmAllergyWidgets.cAllergyManagerDlg(parent=self, id=-1)
 		# FIXME: use signal and use node level update
@@ -633,10 +792,33 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 	def __export_encounter_for_medistar(self, evt):
 		gmNarrativeWidgets.export_narrative_for_medistar_import (
 			parent = self,
-			soap_cats = u'soap',
+			soap_cats = u'soapu',
 			encounter = self.__curr_node_data
 		)
 	#--------------------------------------------------------
+	def __expand_root_node(self):
+		root_node = self.GetRootItem()
+		self.DeleteChildren(root_node)
+
+		issues = [{
+			'description': _('Unattributed episodes'),
+			'has_open_episode': False,
+			'pk_health_issue': None
+		}]
+
+		emr = self.__pat.emr
+		issues.extend(emr.health_issues)
+
+		for issue in issues:
+			issue_node =  self.AppendItem(root_node, issue['description'])
+			self.SetItemBold(issue_node, issue['has_open_episode'])
+			self.SetItemPyData(issue_node, issue)
+			# fake it so we can expand it
+			self.SetItemHasChildren(issue_node, True)
+
+		self.SetItemHasChildren(root_node, (len(issues) != 0))
+		self.SortChildren(root_node)
+	#--------------------------------------------------------
 	# event handlers
 	#--------------------------------------------------------
 	def _on_narrative_mod_db(self, *args, **kwargs):
@@ -648,6 +830,35 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 	def _on_issue_mod_db(self, *args, **kwargs):
 		wx.CallAfter(self.__populate_tree)
 	#--------------------------------------------------------
+	def _on_tree_item_expanding(self, event):
+		if not self.__pat.connected:
+			return
+
+		event.Skip()
+
+		node = event.GetItem()
+		if node == self.GetRootItem():
+			self.__expand_root_node()
+			return
+
+		node_data = self.GetPyData(node)
+
+		if isinstance(node_data, gmEMRStructItems.cHealthIssue):
+			self.__expand_issue_node(issue_node = node)
+			return
+
+		if isinstance(node_data, gmEMRStructItems.cEpisode):
+			self.__expand_episode_node(episode_node = node)
+			return
+
+		# pseudo node "free-standing episodes"
+		if type(node_data) == type({}):
+			self.__expand_pseudo_issue_node(fake_issue_node = node)
+			return
+
+		# encounter nodes do not need expanding
+		#if isinstance(node_data, gmEMRStructItems.cEncounter):
+	#--------------------------------------------------------
 	def _on_tree_item_selected(self, event):
 		sel_item = event.GetItem()
 		self.__curr_node = sel_item
@@ -671,7 +882,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 #					return
 #
 #				self.SetToolTip(u'%s  %s  %s - %s\n\nRFE: %s\nAOE: %s' % (
-#					data['started'].strftime('%x'),
+#					gmDateTime.pydt_strftime(data['started'], '%Y %b %d'),
 #					data['l10n_type'],
 #					data['started'].strftime('%H:%m'),
 #					data['last_affirmed'].strftime('%H:%m'),
@@ -691,7 +902,7 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 
 		if isinstance(data, gmEMRStructItems.cEncounter):
 			tt = u'%s  %s  %s - %s\n' % (
-				data['started'].strftime('%x'),
+				gmDateTime.pydt_strftime(data['started'], '%Y %b %d'),
 				data['l10n_type'],
 				data['started'].strftime('%H:%M'),
 				data['last_affirmed'].strftime('%H:%M')
@@ -869,8 +1080,8 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 
 		# episodes: chronologically
 		if isinstance(item1, gmEMRStructItems.cEpisode):
-			start1 = item1.get_access_range()[0]
-			start2 = item2.get_access_range()[0]
+			start1 = item1.best_guess_start_date
+			start2 = item2.best_guess_start_date
 			if start1 == start2:
 				return 0
 			if start1 < start2:
@@ -922,14 +1133,14 @@ class cEMRTree(wx.TreeCtrl, gmGuiHelpers.cTreeExpansionHistoryMixin):
 	# properties
 	#--------------------------------------------------------
 	def _get_details_display_mode(self):
-		return self.__details_display_mode
+		return self.__soap_display_mode
 
 	def _set_details_display_mode(self, mode):
 		if mode not in [u'details', u'journal']:
 			raise ValueError('details display mode must be one of "details", "journal"')
-		if self.__details_display_mode == mode:
+		if self.__soap_display_mode == mode:
 			return
-		self.__details_display_mode = mode
+		self.__soap_display_mode = mode
 		self.__update_text_for_selected_node()
 
 	details_display_mode = property(_get_details_display_mode, _set_details_display_mode)
@@ -941,7 +1152,7 @@ class cScrolledEMRTreePnl(wxgScrolledEMRTreePnl.wxgScrolledEMRTreePnl):
 
 	Lacks a widget to display details for selected items. The
 	tree data will be refetched - if necessary - whenever
-	repopulate_ui() is called, e.g., when then patient is changed.
+	repopulate_ui() is called, e.g., when the patient is changed.
 	"""
 	def __init__(self, *args, **kwds):
 		wxgScrolledEMRTreePnl.wxgScrolledEMRTreePnl.__init__(self, *args, **kwds)
@@ -962,45 +1173,158 @@ class cSplittedEMRTreeBrowserPnl(wxgSplittedEMRTreeBrowserPnl.wxgSplittedEMRTree
 	"""
 	def __init__(self, *args, **kwds):
 		wxgSplittedEMRTreeBrowserPnl.wxgSplittedEMRTreeBrowserPnl.__init__(self, *args, **kwds)
-		self._pnl_emr_tree._emr_tree.set_narrative_display(narrative_display = self._TCTRL_item_details)
-		self._pnl_emr_tree._emr_tree.set_image_display(image_display = self._PNL_visual_soap)
+		self._pnl_emr_tree._emr_tree.soap_display = self._TCTRL_item_details
+		self._pnl_emr_tree._emr_tree.image_display = self._PNL_visual_soap
 		self._pnl_emr_tree._emr_tree.set_enable_display_mode_selection_callback(self.enable_display_mode_selection)
+		self._pnl_emr_tree._emr_tree.soap_editor_adder = self._add_soap_editor
+		self._pnl_emr_tree._emr_tree.edit_mode_selector = self._select_edit_mode
 		self.__register_events()
+
+		self.editing = False
 	#--------------------------------------------------------
 	def __register_events(self):
+		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		return True
 	#--------------------------------------------------------
+	def _get_editing(self):
+		return self.__editing
+
+	def _set_editing(self, editing):
+		self.__editing = editing
+		self.enable_display_mode_selection(enable = not self.__editing)
+		if self.__editing:
+			self._BTN_switch_browse_edit.SetLabel(_('&Browse'))
+			self._PNL_browse.Hide()
+			self._PNL_visual_soap.Hide()
+			self._PNL_edit.Show()
+		else:
+			self._BTN_switch_browse_edit.SetLabel(_('&Edit'))
+			self._PNL_edit.Hide()
+			self._PNL_visual_soap.Show()
+			self._PNL_browse.Show()
+		self._PNL_right_side.GetSizer().Layout()
+
+	editing = property(_get_editing, _set_editing)
+	#--------------------------------------------------------
 	# event handler
 	#--------------------------------------------------------
-	def _on_post_patient_selection(self):
-		if self.GetParent().GetCurrentPage() == self:
-			self.repopulate_ui()
+	def _on_pre_patient_selection(self):
+		self._pnl_emr_tree._emr_tree.clear_tree()
+		self._PNL_edit.patient = None
 		return True
 	#--------------------------------------------------------
+	def _on_post_patient_selection(self):
+		wx.CallAfter(self.__on_post_patient_selection)
+		return True
+	#--------------------------------------------------------
+	def __on_post_patient_selection(self):
+		if self.GetParent().GetCurrentPage() != self:
+			return True
+		self.repopulate_ui()
+	#--------------------------------------------------------
 	def _on_show_details_selected(self, event):
-		#event.Skip()
 		self._pnl_emr_tree._emr_tree.details_display_mode = u'details'
 	#--------------------------------------------------------
 	def _on_show_journal_selected(self, event):
-		#event.Skip()
 		self._pnl_emr_tree._emr_tree.details_display_mode = u'journal'
 	#--------------------------------------------------------
+	def _on_switch_browse_edit_button_pressed(self, event):
+		self.editing = not self.__editing
+	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
 	def repopulate_ui(self):
 		"""Fills UI with data."""
 		self._pnl_emr_tree.repopulate_ui()
+		self._PNL_edit.patient = gmPerson.gmCurrentPatient()
 		self._splitter_browser.SetSashPosition(self._splitter_browser.GetSizeTuple()[0]/3, True)
 		return True
 	#--------------------------------------------------------
 	def enable_display_mode_selection(self, enable):
+		if self.editing:
+			enable = False
 		if enable:
 			self._RBTN_details.Enable(True)
 			self._RBTN_journal.Enable(True)
 			return
 		self._RBTN_details.Enable(False)
 		self._RBTN_journal.Enable(False)
+	#--------------------------------------------------------
+	def _add_soap_editor(self, problem=None, allow_same_problem=False):
+		self._PNL_edit._NB_soap_editors.add_editor(problem = problem, allow_same_problem = allow_same_problem)
+	#--------------------------------------------------------
+	def _select_edit_mode(self, edit=True):
+		self.editing = edit
+
+#================================================================
+from Gnumed.wxGladeWidgets import wxgEMRJournalPluginPnl
+
+class cEMRJournalPluginPnl(wxgEMRJournalPluginPnl.wxgEMRJournalPluginPnl):
+
+	def __init__(self, *args, **kwds):
+
+		wxgEMRJournalPluginPnl.wxgEMRJournalPluginPnl.__init__(self, *args, **kwds)
+		self._TCTRL_journal.SetValue(u'')
+	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
+	def repopulate_ui(self):
+		self._TCTRL_journal.SetValue(u'')
+		exporter = gmPatientExporter.cEMRJournalExporter()
+		if self._RBTN_by_encounter.GetValue():
+			txt = StringIO.StringIO()
+			# FIXME: if journal is large this will error out, use generator/yield etc
+			# FIXME: turn into proper list
+			try:
+				exporter.export(txt)
+				self._TCTRL_journal.SetValue(txt.getvalue())
+			except ValueError:
+				_log.exception('cannot get EMR journal')
+				self._TCTRL_journal.SetValue (_(
+					'An error occurred while retrieving the EMR\n'
+					'in journal form for the active patient.\n\n'
+					'Please check the log file for details.'
+				))
+			txt.close()
+		else:
+			fname = exporter.export_to_file_by_mod_time()
+			f = codecs.open(filename = fname, mode = 'rU', encoding = 'utf8', errors = 'replace')
+			for line in f:
+				self._TCTRL_journal.AppendText(line)
+			f.close()
+
+		self._TCTRL_journal.ShowPosition(self._TCTRL_journal.GetLastPosition())
+		return True
+	#--------------------------------------------------------
+	# internal helpers
+	#--------------------------------------------------------
+	def __register_events(self):
+		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
+		return True
+	#--------------------------------------------------------
+	# event handler
+	#--------------------------------------------------------
+	def _on_pre_patient_selection(self):
+		self._TCTRL_journal.SetValue(u'')
+		return True
+	#--------------------------------------------------------
+	def _on_post_patient_selection(self):
+		wx.CallAfter(self.__on_post_patient_selection)
+		return True
+	#--------------------------------------------------------
+	def __on_post_patient_selection(self):
+		if self.GetParent().GetCurrentPage() != self:
+			return True
+		self.repopulate_ui()
+	#--------------------------------------------------------
+	def _on_order_by_encounter_selected(self, event):
+		self.repopulate_ui()
+	#--------------------------------------------------------
+	def _on_order_by_last_mod_selected(self, event):
+		self.repopulate_ui()
+
 #================================================================
 class cEMRJournalPanel(wx.Panel):
 	def __init__(self, *args, **kwargs):
@@ -1039,21 +1363,28 @@ class cEMRJournalPanel(wx.Panel):
 	# notebook plugin API
 	#--------------------------------------------------------
 	def repopulate_ui(self):
-		txt = StringIO.StringIO()
+#		txt = StringIO.StringIO()
 		exporter = gmPatientExporter.cEMRJournalExporter()
-		# FIXME: if journal is large this will error out, use generator/yield etc
-		# FIXME: turn into proper list
-		try:
-			exporter.export(txt)
-			self.__journal.SetValue(txt.getvalue())
-		except ValueError:
-			_log.exception('cannot get EMR journal')
-			self.__journal.SetValue (_(
-				'An error occurred while retrieving the EMR\n'
-				'in journal form for the active patient.\n\n'
-				'Please check the log file for details.'
-			))
-		txt.close()
+		fname = exporter.export_to_file_by_mod_time()
+		f = codecs.open(filename = fname, mode = 'rU', encoding = 'utf8', errors = 'replace')
+		for line in f:
+			self.__journal.AppendText(line)
+		f.close()
+
+#		# FIXME: if journal is large this will error out, use generator/yield etc
+#		# FIXME: turn into proper list
+#		try:
+#			exporter.export(txt)
+#			self.__journal.SetValue(txt.getvalue())
+#		except ValueError:
+#			_log.exception('cannot get EMR journal')
+#			self.__journal.SetValue (_(
+#				'An error occurred while retrieving the EMR\n'
+#				'in journal form for the active patient.\n\n'
+#				'Please check the log file for details.'
+#			))
+#		txt.close()
+
 		self.__journal.ShowPosition(self.__journal.GetLastPosition())
 		return True
 #================================================================
